@@ -1,14 +1,14 @@
 CREATE TYPE pgroonga_full_text_search_condition AS (
-  query text,
-  weigths int[],
-  indexName text
+	query text,
+	weigths int[],
+	indexName text
 );
 
 CREATE TYPE pgroonga_full_text_search_condition_with_scorers AS (
-  query text,
-  weigths int[],
-  scorers text[],
-  indexName text
+	query text,
+	weigths int[],
+	scorers text[],
+	indexName text
 );
 
 CREATE FUNCTION pgroonga_score("row" record)
@@ -361,6 +361,46 @@ CREATE FUNCTION pgroonga_result_to_jsonb_objects(result jsonb)
 	STRICT
 	PARALLEL SAFE;
 
+CREATE FUNCTION pgroonga_wal_status()
+	RETURNS TABLE(
+			name text,
+			oid oid,
+			current_block int8,
+			current_offset int8,
+			current_size int8,
+			last_block int8,
+			last_offset int8,
+			last_size int8
+		)
+	AS 'MODULE_PATHNAME', 'pgroonga_wal_status'
+	LANGUAGE C
+	STRICT
+	PARALLEL SAFE;
+
+CREATE FUNCTION pgroonga_wal_set_applied_position(indexName cstring, "block" bigint, "offset" bigint)
+	RETURNS bool
+	AS 'MODULE_PATHNAME', 'pgroonga_wal_set_applied_position_index'
+	LANGUAGE C
+	STRICT;
+
+CREATE FUNCTION pgroonga_wal_set_applied_position(indexName cstring)
+	RETURNS bool
+	AS 'MODULE_PATHNAME', 'pgroonga_wal_set_applied_position_index_last'
+	LANGUAGE C
+	STRICT;
+
+CREATE FUNCTION pgroonga_wal_set_applied_position("block" bigint, "offset" bigint)
+	RETURNS bool
+	AS 'MODULE_PATHNAME', 'pgroonga_wal_set_applied_position_all'
+	LANGUAGE C
+	STRICT;
+
+CREATE FUNCTION pgroonga_wal_set_applied_position()
+	RETURNS bool
+	AS 'MODULE_PATHNAME', 'pgroonga_wal_set_applied_position_all_last'
+	LANGUAGE C
+	STRICT;
+
 
 /* v1 */
 CREATE FUNCTION pgroonga_match_term(target text, term text)
@@ -1794,6 +1834,25 @@ CREATE OPERATOR &=~ (
 	JOIN = contjoinsel
 );
 
+CREATE FUNCTION pgroonga_equal_query_text_array_condition
+	(targets text[], condition pgroonga_full_text_search_condition)
+	RETURNS bool
+	AS 'MODULE_PATHNAME', 'pgroonga_equal_query_text_array_condition'
+	LANGUAGE C
+	IMMUTABLE
+	STRICT
+	LEAKPROOF
+	PARALLEL SAFE
+	COST 300;
+
+CREATE OPERATOR &=~ (
+	PROCEDURE = pgroonga_equal_query_text_array_condition,
+	LEFTARG = text[],
+	RIGHTARG = pgroonga_full_text_search_condition,
+	RESTRICT = contsel,
+	JOIN = contjoinsel
+);
+
 CREATE FUNCTION pgroonga_equal_query_varchar_array(targets varchar[], query text)
 	RETURNS bool
 	AS 'MODULE_PATHNAME', 'pgroonga_equal_query_varchar_array'
@@ -1812,6 +1871,25 @@ CREATE OPERATOR &=~ (
 	JOIN = contjoinsel
 );
 
+CREATE FUNCTION pgroonga_equal_query_varchar_array_condition
+	(targets varchar[], condition pgroonga_full_text_search_condition)
+	RETURNS bool
+	AS 'MODULE_PATHNAME', 'pgroonga_equal_query_varchar_array_condition'
+	LANGUAGE C
+	IMMUTABLE
+	STRICT
+	LEAKPROOF
+	PARALLEL SAFE
+	COST 300;
+
+CREATE OPERATOR &=~ (
+	PROCEDURE = pgroonga_equal_query_varchar_array_condition,
+	LEFTARG = varchar[],
+	RIGHTARG = pgroonga_full_text_search_condition,
+	RESTRICT = contsel,
+	JOIN = contjoinsel
+);
+
 
 DROP ACCESS METHOD IF EXISTS pgroonga CASCADE;
 CREATE FUNCTION pgroonga_handler(internal)
@@ -2045,7 +2123,8 @@ CREATE OPERATOR CLASS pgroonga_text_array_term_search_ops_v2 FOR TYPE text[]
 		OPERATOR 21 &^~| (text[], text[]),
 		OPERATOR 24 &^> (text[], text), -- For backward compatibility
 		OPERATOR 25 &^~> (text[], text), -- For backward compatibility
-		OPERATOR 40 &=~ (text[], text);
+		OPERATOR 40 &=~ (text[], text),
+		OPERATOR 41 &=~ (text[], pgroonga_full_text_search_condition);
 
 CREATE OPERATOR CLASS pgroonga_text_regexp_ops_v2 FOR TYPE text
 	USING pgroonga AS
@@ -2102,7 +2181,8 @@ CREATE OPERATOR CLASS pgroonga_varchar_array_term_search_ops_v2
 		OPERATOR 23 &> (varchar[], varchar),
 		OPERATOR 24 &^> (varchar[], varchar), -- For backward compatibility
 		OPERATOR 25 &^~> (varchar[], varchar), -- For backward compatibility
-		OPERATOR 40 &=~ (varchar[], text);
+		OPERATOR 40 &=~ (varchar[], text),
+		OPERATOR 41 &=~ (varchar[], pgroonga_full_text_search_condition);
 
 CREATE OPERATOR CLASS pgroonga_varchar_regexp_ops_v2 FOR TYPE varchar
 	USING pgroonga AS
@@ -2127,6 +2207,7 @@ CREATE OPERATOR CLASS pgroonga_jsonb_full_text_search_ops_v2
 		OPERATOR 28 &@~ (jsonb, text);
 
 -- For backward compatibility
+-- TODO: We must remove this when we release 4.0.0.
 
 DO LANGUAGE plpgsql $$
 BEGIN
